﻿section SynergyBI;

//
// Definition
//

// Data Source Kind description
SynergyBI = [
     TestConnection = (orgSlug) => { "Test1.Contents", orgSlug },
    Authentication = [
        Key = [
            KeyLabel = "ApiKey",
            Label = "apikey1"
        ],
        UsernamePassword = []
    ],
    Label = "SynergyBI"
];

// Data Source UI publishing description
SynergyBI.Publish = [
    Beta = true,
    Category = "Other",
    ButtonText = { "SynergyBI Navigator", "SynergyBI Navigator" }
];

//
// Implementation
// 


[DataSource.Kind="SynergyBI", Publish="SynergyBI.Publish"]
shared SynergyBI.Contents = Value.ReplaceType(SynergyBINavTable, type function (orgSlug as Uri.Type) as any);

shared Test1.Contents = (arg) => 
    let 
        source = 1
    in 
        source;

DefaultRequestHeaders = [
    #"Accept" = "application/json;odata.metadata=minimal",  // column name and values only
    #"OData-MaxVersion" = "4.0"                             // we only support v4
];

//[DataSource.Kind="SynergyBI"]
shared SynergyBI.Feed = Value.ReplaceType(SynergyBIImpl, type function (orgSlug as Uri.Type) as any);

SynergyBIImpl = (url1 as text) =>
    let
        source = Web.Contents(url1, [ Headers = DefaultRequestHeaders ]),
        json = Json.Document(source)
    in
        json;

SynergyBINavTable = (orgSlug as text) as table =>
    let
        source = #table({"Name", "Data", "ItemKind", "ItemName", "IsLeaf"}, {
            { "Projects", GetProjectsTable(orgSlug), "Table", "Table", true },
            { "Stages", GetStagesTable(orgSlug), "Table", "Table", true },
            { "Transactions", GetTransactionsTable(orgSlug), "Table", "Table", true },
            { "ProjectStatusChanges", GetProjectsStatusesChanges(orgSlug), "Table", "Table", true },
            { "Contacts", GetContactsTable(orgSlug), "Table", "Table", true },
            { "StageStatusChanges", GetStagesStatusesChanges(orgSlug), "Table", "Table", true },
            { "Invoices", GetInvoicesTable(orgSlug), "Table", "Table", true },
            { "InvoicePayments", GetInvoicesPaymentsTable(orgSlug), "Table", "Table", true },
            { "Overtime", GetOvertimeTable(orgSlug), "Table", "Table", true },
            { "ProjectContacts", GetProjectContactsTable(orgSlug), "Table", "Table", true },
            { "Rates", GetRates(orgSlug), "Table", "Table", true },
            { "Tasks", GetTasks(orgSlug), "Table", "Table", true }
        }),
        navTable = Table.ToNavigationTable(source, {"Name"}, "Name", "Data", "ItemKind", "ItemName", "IsLeaf")
    in
        navTable;


//           { "Staff", GetStaffTable(orgSlug), "Table", "Table", true },


GetJson = (usedRelativeUrl) =>
        let  options = [Headers = [#"IGNORE_ME_Content-TypeTT"="application/json",
            #"access-token"=Extension.CurrentCredential()[Key]],
            RelativePath=usedRelativeUrl
            ],
            RawData = Web.Contents("app.totalsynergy.com", options),
            Json    = Json.Document(RawData)
        in  Json;

GetEntityCount = (relativeUrl) => 
        let relativeUrl1 =
            Text.Replace(relativeUrl,"##", "0"),
            Json  = GetJson(relativeUrl1),
            Count = Json[#"totalPages"]
        in  Count;

GetPage = (relativeUrl, Index) =>
        let relativeUrl1 =
            Text.Replace(relativeUrl,"##", Text.From(Index)),
            Json  = GetJson(relativeUrl1),
            Value = Json[#"items"]
        in  Value;

GetAllPages = (relativeUrl) =>
        let 
            MAX_PAGES=1000,
              pages = List.Min({ MAX_PAGES, GetEntityCount(relativeUrl) }),
            PageCount   = pages,
            PageIndices = { 1 .. PageCount },
            Pages       = List.Transform(PageIndices, each GetPage(relativeUrl,_)),
            Entities    = List.Combine(Pages)
        in  Entities;


GetContactsTable = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/contacts/" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    Entities1 = Entities{0},
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "contactTypeId", "organisationId", "organisation", "avatar", "isStaff", "staffTypeName", "phones", "timezoneId", "userCurrentDate", "rolesDisplay", "roleIds", "emailDomain", "web", "jobTitle", "name", "initials", "localTime", "salutationId", "salutation", "firstName", "lastName", "email", "cardId", "staffCardId", "hasAddresses", "addressHome", "addressWork", "addressPostal", "addressOther", "typeColumnDisplay", "typeDisplay", "website", "phoneDisplay", "profileImageURL", "email2", "email3", "email4", "isActive", "abn", "acn", "contactStatusId", "contactStatus", "accountManagerId", "accountManagerName", "accountCode", "subContactTypeId", "parentContactId", "parentContactName", "tags", "position", "createdDate", "updatedDate"}, {"id", "contactTypeId", "organisationId", "organisation", "avatar", "isStaff", "staffTypeName", "phones", "timezoneId", "userCurrentDate", "rolesDisplay", "roleIds", "emailDomain", "web", "jobTitle", "name", "initials", "localTime", "salutationId", "salutation", "firstName", "lastName", "email", "cardId", "staffCardId", "hasAddresses", "addressHome", "addressWork", "addressPostal", "addressOther", "typeColumnDisplay", "typeDisplay", "website", "phoneDisplay", "profileImageURL", "email2", "email3", "email4", "isActive", "abn", "acn", "contactStatusId", "contactStatus", "accountManagerId", "accountManagerName", "accountCode", "subContactTypeId", "parentContactId", "parentContactName", "tags", "position", "createdDate", "updatedDate"})
in
    #"Expanded Column1";

GetProjectsTable = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/projects/" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "organisationId", "organisationName", "isOfficeProject", "isDefault", "number", "name", "scope", "statusName", "startDate", "endDate", "invoices", "isActive", "createdDate", "updatedDate", "address", "primaryContactId", "primaryContactName", "customFields"}, {"Column1.id", "Column1.organisationId", "Column1.organisationName", "Column1.isOfficeProject", "Column1.isDefault", "Column1.number", "Column1.name", "Column1.scope", "Column1.statusName", "Column1.startDate", "Column1.endDate", "Column1.invoices", "Column1.isActive", "Column1.createdDate", "Column1.updatedDate", "Column1.address", "Column1.primaryContactId", "Column1.primaryContactName", "Column1.customFields"}),
    #"Expanded Column1.address" = Table.ExpandRecordColumn(#"Expanded Column1", "Column1.address", {"id", "address1", "address2", "town", "state", "zipCode", "country", "latitude", "longitude", "hasAddress", "singleline", "googleAddress", "stateZipCountry"}, {"Column1.address.id", "Column1.address.address1", "Column1.address.address2", "Column1.address.town", "Column1.address.state", "Column1.address.zipCode", "Column1.address.country", "Column1.address.latitude", "Column1.address.longitude", "Column1.address.hasAddress", "Column1.address.singleline", "Column1.address.googleAddress", "Column1.address.stateZipCountry"})
in
    #"Expanded Column1.address";

// GetStaffTable = (orgSlug as text) as table => let 
//     relativeUrl     = "api/v2/Organisation/" & orgSlug & "/staff/" & "?pageSize=500" & "&page=##",
//     Entities    = GetAllPages(relativeUrl),
//     Entities1 = Entities{0},
//     #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
//     #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"reportsTo", "addressHome", "addressWork", "addressPostal", "addressOther", "phones", "id", "organisationId", "name", "lastName", "firstName", "email", "email2", "email3", "email4", "contactTypeId", "cardId", "dateEmployment", "dateTermination", "createdDate", "updatedDate", "position", "formalTitle", "rate", "rateDisplay", "isActive", "twitter", "payRate", "payType", "payTypeId", "isPaidPerHour", "payRateEffectiveDate"}, {"reportsTo", "addressHome", "addressWork", "addressPostal", "addressOther", "phones", "id", "organisationId", "name", "lastName", "firstName", "email", "email2", "email3", "email4", "contactTypeId", "cardId", "dateEmployment", "dateTermination", "createdDate", "updatedDate", "position", "formalTitle", "rate", "rateDisplay", "isActive", "twitter", "payRate", "payType", "payTypeId", "isPaidPerHour", "payRateEffectiveDate"})
//  in
//     #"Expanded Column1";

GetStagesTable = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/Stages/" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    Entities1 = Entities{0},
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "organisationId", "name", "statusId", "status", "statusTypeId", "feeTypeId", "feeType", "wip", "fee", "timeSheetsPending", "managerId", "manager", "startDate", "endDate", "updatedDate", "projectId", "costCentre", "budget", "agreedFeeAndCap", "netInvoicedTotalExTax", "netInvoicedTotalIncTax", "invoicedTotalExTax", "invoicedTotalIncTax", "adjustedTotalExTax", "adjustedTotalIncTax", "totalWIPValue", "totalChargable", "residual", "netResidual", "remainingWIPValue", "writtenOffValue", "transactions", "tasks", "disipline"}, {"id", "organisationId", "name", "statusId", "status", "statusTypeId", "feeTypeId", "feeType", "wip", "fee", "timeSheetsPending", "managerId", "manager", "startDate", "endDate", "updatedDate", "projectId", "costCentre", "budget", "agreedFeeAndCap", "netInvoicedTotalExTax", "netInvoicedTotalIncTax", "invoicedTotalExTax", "invoicedTotalIncTax", "adjustedTotalExTax", "adjustedTotalIncTax", "totalWIPValue", "totalChargable", "residual", "netResidual", "remainingWIPValue", "writtenOffValue", "transactions", "tasks", "disipline"})
in
    #"Expanded Column1";

GetTransactionsTable = (orgSlug as text) as table => let 
    startDate=Date.AddMonths(DateTime.LocalNow(),-4),
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/Transactions/bulkexport/" & "?pageSize=500" & "&DateRange.From=" & DateTime.ToText(startDate, "yyyy/MM/dd") & "&DateRange.To=" & DateTime.ToText(DateTime.LocalNow(), "yyyy/MM/dd") & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "projectId", "transactionTypeId", "statusId", "status", "date", "note", "costIncTax", "cost", "actualCost", "standardCost", "actualCostTotal", "standardCostTotal", "targetCharge", "targetChargeTotal", "units", "value", "valueTotal", "wipValueTotal", "potentialValue", "potentialValueTotal", "isUtilised", "revenueMarkupExTax", "netInvoiceExTax", "netInvoiceTotalExTax", "revenueExTax", "revenueTotalExTax", "invoiceNumber", "invoiceDate", "invoiceValue", "invoiceValueTotal", "invoiceOriginalValue", "invoiceOriginalValueTotal", "invoiceUnits", "reasonComment", "createdBy", "createdDate", "updatedBy", "updatedDate", "stageId", "taskId", "staffId", "stageName", "projectNumber", "projectName", "taskName"}, {"id", "projectId", "transactionTypeId", "statusId", "status", "date", "note", "costIncTax", "cost", "actualCost", "standardCost", "actualCostTotal", "standardCostTotal", "targetCharge", "targetChargeTotal", "units", "value", "valueTotal", "wipValueTotal", "potentialValue", "potentialValueTotal", "isUtilised", "revenueMarkupExTax", "netInvoiceExTax", "netInvoiceTotalExTax", "revenueExTax", "revenueTotalExTax", "invoiceNumber", "invoiceDate", "invoiceValue", "invoiceValueTotal", "invoiceOriginalValue", "invoiceOriginalValueTotal", "invoiceUnits", "reasonComment", "createdBy", "createdDate", "updatedBy", "updatedDate", "stageId", "taskId", "staffId", "stageName", "projectNumber", "projectName", "taskName"})
in
    #"Expanded Column1";


GetProjectsStatusesChanges = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/ProjectStatusChanges/" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"fromId", "fromName", "fromTypeId", "fromTypeName", "toId", "toName", "toTypeId", "toTypeName", "dateTimeChanged", "projectId", "projectName"}, {"fromId", "fromName", "fromTypeId", "fromTypeName", "toId", "toName", "toTypeId", "toTypeName", "dateTimeChanged", "projectId", "projectName"})
in
    #"Expanded Column1";

GetStagesStatusesChanges = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/StageStatusChanges/" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    Entities1 = Entities{0},
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"fromId", "fromName", "fromTypeId", "fromTypeName", "toId", "toName", "toTypeId", "toTypeName", "dateTimeChanged", "projectId", "projectName", "stageId", "stageName"}, {"fromId", "fromName", "fromTypeId", "fromTypeName", "toId", "toName", "toTypeId", "toTypeName", "dateTimeChanged", "projectId", "projectName", "stageId", "stageName"})
in
    #"Expanded Column1";

GetInvoicesTable = (orgSlug as text) as table => let 
    startDate=Date.AddMonths(DateTime.LocalNow(),-4),
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/invoices/index/" & "?pageSize=500" & "&CreatedDateFrom=" & DateTime.ToText(startDate, "yyyy/MM/dd") & "&CreatedDateTo=" & DateTime.ToText(DateTime.LocalNow(), "yyyy/MM/dd") & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"dateFormat", "currencySymbol", "id", "organisationId", "projectId", "projectNumber", "projectName", "status", "statusDisplay", "invoiceNumber", "secondaryNumber", "invoiceNumberDisplay", "numberingId", "prefix", "number", "numberFormatted", "invoiceType", "text", "totalExTax", "totalExTaxDisplay", "totalTax", "totalTaxDisplay", "totalIncTax", "totalIncTaxDisplay", "owingIncTax", "owingIncTaxDisplay", "paymentsIncTax", "paymentsIncTaxDisplay", "agedDebtor", "invoiceContactId", "invoiceContact", "primaryContactId", "primaryContact", "date", "dateDisplay", "from", "fromDisplay", "to", "toDisplay", "dueDate", "dueDateDisplay", "invoiceTermId", "invoiceTerm", "invoiceStages", "invoiceContacts", "projectContactIds", "finalize", "parentInvoiceId"}, {"dateFormat", "currencySymbol", "id", "organisationId", "projectId", "projectNumber", "projectName", "status", "statusDisplay", "invoiceNumber", "secondaryNumber", "invoiceNumberDisplay", "numberingId", "prefix", "number", "numberFormatted", "invoiceType", "text", "totalExTax", "totalExTaxDisplay", "totalTax", "totalTaxDisplay", "totalIncTax", "totalIncTaxDisplay", "owingIncTax", "owingIncTaxDisplay", "paymentsIncTax", "paymentsIncTaxDisplay", "agedDebtor", "invoiceContactId", "invoiceContact", "primaryContactId", "primaryContact", "date", "dateDisplay", "from", "fromDisplay", "to", "toDisplay", "dueDate", "dueDateDisplay", "invoiceTermId", "invoiceTerm", "invoiceStages", "invoiceContacts", "projectContactIds", "finalize", "parentInvoiceId"}),
    #"Changed Type" = Table.TransformColumnTypes(#"Expanded Column1",{{"totalIncTax", type number}, {"date", type datetime}})
in
    #"Changed Type";

GetInvoicesPaymentsTable = (orgSlug as text) as table => let 
    startDate=Date.AddMonths(DateTime.LocalNow(),-12),
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/InvoicePayments/index/" & "?pageSize=500" & "&CreatedDate=" & DateTime.ToText(startDate, "yyyy/MM/dd") & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "invoiceId", "invoiceNumber", "projectId", "projectNumber", "projectTitle", "date", "dateAsInt", "dateFormat", "dateDisplay", "note", "paymentTypeId", "paymentType", "amount", "amountDisplay", "owing", "owingDisplay", "owingFormatted", "currencySymbol", "createdBy", "createdByName", "createdDate", "createdByDisplay", "updatedDate", "updatedBy", "error"}, {"id", "invoiceId", "invoiceNumber", "projectId", "projectNumber", "projectTitle", "date", "dateAsInt", "dateFormat", "dateDisplay", "note", "paymentTypeId", "paymentType", "amount", "amountDisplay", "owing", "owingDisplay", "owingFormatted", "currencySymbol", "createdBy", "createdByName", "createdDate", "createdByDisplay", "updatedDate", "updatedBy", "error"})
in
    #"Expanded Column1";

GetOvertimeTable = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/overtimes/index" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "name", "rateFactor", "costFactor", "isActive", "isDefault", "createdBy", "createdDate", "updatedBy", "updatedDate"}, {"id", "name", "rateFactor", "costFactor", "isActive", "isDefault", "createdBy", "createdDate", "updatedBy", "updatedDate"})
in
    #"Expanded Column1";

GetProjectContactsTable = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/ProjectContacts/Index" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "contactId", "name", "contactTypeId", "roleDisplay", "isActive", "projectId", "projectNameDisplay", "updatedDate", "createdDate", "contact", "isManager", "isPrimary", "isInvoice", "isCareOf"}, {"id", "contactId", "name", "contactTypeId", "roleDisplay", "isActive", "projectId", "projectNameDisplay", "updatedDate", "createdDate", "contact", "isManager", "isPrimary", "isInvoice", "isCareOf"})
in
    #"Expanded Column1";

GetRates = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/Rates/index" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    Entities1 = Entities{0},
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "organisationId", "name", "typeId", "typeName", "isStaff", "isMarkup", "createdBy", "createdDate", "updatedBy", "updatedDate", "rateCategory", "rateCategoryId"}, {"id", "organisationId", "name", "typeId", "typeName", "isStaff", "isMarkup", "createdBy", "createdDate", "updatedBy", "updatedDate", "rateCategory", "rateCategoryId"})
in
    #"Expanded Column1";

GetTasks = (orgSlug as text) as table => let 
    relativeUrl     = "api/v2/Organisation/" & orgSlug & "/Tasks/" & "?pageSize=500" & "&page=##",
    Entities    = GetAllPages(relativeUrl),
    Entities1 = Entities{0},
    #"Converted to Table" = Table.FromList(Entities, Splitter.SplitByNothing(), null, null, ExtraValues.Error),
    #"Expanded Column1" = Table.ExpandRecordColumn(#"Converted to Table", "Column1", {"id", "projectId", "stageId", "stageName", "workBreakdownId", "sort", "number", "expenseTypeId", "expenseType", "completePercent", "name", "note", "canDelete", "canChangetask", "percent", "color", "startDate", "endDate", "createdDate", "completedDate", "createdBy", "updatedDate", "updatedBy", "updatedDateAsInt", "startDateAsInt", "endDateAsInt", "createdDateAsInt", "completedDateAsInt", "blockAllDays", "budgetAmount", "budgetUnits", "targetSortPositionTaskId", "tags", "position", "stagePosition", "phasePosition", "unitSummary", "totalExTaxSummary", "noteCount", "transactionCount", "budgetCount", "unAllocatedHours", "isTimerRunning", "delete", "isLocked", "isTemplate", "isUtilised", "timesheetTaskId", "type", "taskRates", "staff", "budgets", "notes", "projectName", "projectAddress", "projectNumber", "phaseId", "phaseName", "enablePhases", "showCompleted", "completedTodoCount", "totalTodoCount", "commandType", "requiredStageIds", "resourceStageIds", "tableViewStageIds", "filterOptions"}, {"id", "projectId", "stageId", "stageName", "workBreakdownId", "sort", "number", "expenseTypeId", "expenseType", "completePercent", "name", "note", "canDelete", "canChangetask", "percent", "color", "startDate", "endDate", "createdDate", "completedDate", "createdBy", "updatedDate", "updatedBy", "updatedDateAsInt", "startDateAsInt", "endDateAsInt", "createdDateAsInt", "completedDateAsInt", "blockAllDays", "budgetAmount", "budgetUnits", "targetSortPositionTaskId", "tags", "position", "stagePosition", "phasePosition", "unitSummary", "totalExTaxSummary", "noteCount", "transactionCount", "budgetCount", "unAllocatedHours", "isTimerRunning", "delete", "isLocked", "isTemplate", "isUtilised", "timesheetTaskId", "type", "taskRates", "staff", "budgets", "notes", "projectName", "projectAddress", "projectNumber", "phaseId", "phaseName", "enablePhases", "showCompleted", "completedTodoCount", "totalTodoCount", "commandType", "requiredStageIds", "resourceStageIds", "tableViewStageIds", "filterOptions"})
in
    #"Expanded Column1";

//
// Common functions
//
Table.ToNavigationTable = (
    table as table,
    keyColumns as list,
    nameColumn as text,
    dataColumn as text,
    itemKindColumn as text,
    itemNameColumn as text,
    isLeafColumn as text
) as table =>
    let
        tableType = Value.Type(table),
        newTableType = Type.AddTableKey(tableType, keyColumns, true) meta 
        [
            NavigationTable.NameColumn = nameColumn, 
            NavigationTable.DataColumn = dataColumn,
            NavigationTable.ItemKindColumn = itemKindColumn, 
            Preview.DelayColumn = itemNameColumn, 
            NavigationTable.IsLeafColumn = isLeafColumn
        ],
        navigationTable = Value.ReplaceType(table, newTableType)
    in
        navigationTable;